@@ -7,22 +7,24 @@ my $c = 0;
 my $conn = new Jabber::JAX::Component(
       component   => "echocomp",
       secret      => "mysecret",
-      host        => "localhost",
+      host        => "localhost.localdomain",
       port        => "7000",
       handler     =>
 
       sub {
             my ( $rc, $p ) = @_;
-	    print STDERR "Doint number ".$c++."\n";
+	    print STDERR "Doing number ".$c++."\n";
             my $e = $p->getElement();
             my $to = $e->getAttrib('to');
             $e->putAttrib('to', $e->getAttrib('from'));
             $e->putAttrib('from', $to);
+	    print STDERR "packet is: ".$e->toString()."\n";
             $rc->deliver( $p );
-            $rc->stop() if $c == 50;
+            $rc->stop() if $c > 2;
 
           }
      );
 
 $conn->start();
 
+print "finished!!!\n";
